//+------------------------------------------------------------------+
//|                                                    RSI.mq5       |
//|                 Copyright 2010, Programmed by onestepremoved.com |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Copyright 2010, Programmed by onestepremoved.com"
#property link      ""
#property version   "1.00"

#include <Trade/Trade.mqh>

#define DEVIATION_PIPS 2

//--- input parameters
input uchar    RSI_Upper      = 73;
input uchar    RSI_Lower      = 27;
input uchar    RSI_Period     = 2;
input short    StopLoss       = 36;
input short    TakeProfit     = 12;
input ushort   Magic          = 12345;
input double   Lot            = 0.1;

// global variables
short stopLoss;
short takeProfit;
int rsiHandle;  // handle for our Relative Strength indicator
double rsiVal[1]; // Dynamic array to hold the values of Relative Strength for each bars

//+------------------------------------------------------------------+
//| Expert initialization function                                   |
//+------------------------------------------------------------------+
int OnInit()
  {
   // Adjust stop loss and take profit for fractional pip brokers
   
   if(_Digits==5 || _Digits==3)
   {
      stopLoss = StopLoss * 10;
      takeProfit = TakeProfit * 10;
   }


   //--- Get the handle for Relative Strength indicator
   
   rsiHandle = iRSI(_Symbol, _Period, RSI_Period, PRICE_CLOSE);

   //--- What if handle returns Invalid Handle
   if(rsiHandle<0)
   {
      Alert("Error Creating Handles for indicators - error: ",GetLastError(),"!!");
   }  
   
   /* Make sure array values for the RSI is stored serially similar to the 
    timeseries array. 
    */
   ArraySetAsSeries(rsiVal,true);
       
//---
   return(0);
  }
//+------------------------------------------------------------------+
//| Expert deinitialization function                                 |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
//---
   IndicatorRelease(rsiHandle);
   
  }
//+------------------------------------------------------------------+
//| Expert tick function                                             |
//+------------------------------------------------------------------+
void OnTick()
  {
   bool bullish = false;
   bool bearish = false;
   
   
   // Only run every bar.
   //--------------------

   static datetime Old_Time;
   datetime New_Time[1];
   bool IsNewBar=false;
   
   // copying the last bar time to the element New_Time[0]
   int copied=CopyTime(_Symbol,_Period,0,1,New_Time);
   if(copied>0) // ok, the data has been copied successfully
     {
      if(Old_Time!=New_Time[0]) // if old time isn't equal to new bar time
        {
         IsNewBar=true;   // if it isn't a first call, the new bar has appeared
         if(MQL5InfoInteger(MQL5_DEBUGGING)) 
            Print("We have new bar here ",New_Time[0]," old time was ",Old_Time);
         Old_Time=New_Time[0];            // saving bar time
        }
     }
   else
     {
      Alert("Error in copying historical times data, error =",GetLastError());
      ResetLastError();
      return;
     }

   //--- EA should only check for new trade if we have a new bar
   if(IsNewBar==false)
   {
      return;
   }
   
   //--- Define some MQL5 Structures we will use for our trade
   MqlTick latest_price;      // To be used for getting recent/latest price quotes
   MqlTradeRequest mrequest;  // To be used for sending our trade requests
   MqlTradeResult mresult;    // To be used to get our trade results
   
   // Get data.
   //----------
   
   // Get Relative Strength
   if(CopyBuffer(rsiHandle,0,0,1,rsiVal)<0)
   {
      Alert("Error copying Relative Strength indicator buffer - error:",GetLastError());
      return;
   }   
   double RSI = rsiVal[0];
//   Print("RSI: ", RSI);

   //--- Get the last price quote using the MQL5 MqlTick Structure
   if(!SymbolInfoTick(_Symbol,latest_price))
   {
      Alert("Error getting the latest price quote - error:",GetLastError(),"!!");
      return;
   }   
   
   
   // See if we have an entry condition.
   //-----------------------------------
   
   if (RSI < RSI_Lower)
   {
      bullish = true;
//      Print("we're bullish");
   }
      
   else if (RSI > RSI_Upper)
   {
      bearish = true;
//      Print("we're bearish");
   }
      

   // Close opposite direction trades.
   //---------------------------------
   
   //--- First, Do we have positions opened already?
    bool Buy_opened=false;  // variable to hold the result of Buy opened position
    bool Sell_opened=false; // variable to hold the result of Sell opened position
    
    if (PositionSelect(_Symbol) ==true)  // we have an opened position
    {
         if (PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_BUY)
         {
            Buy_opened = true;  //It is a Buy
         }
         else if(PositionGetInteger(POSITION_TYPE) == POSITION_TYPE_SELL)
         {
            Sell_opened = true; // It is a Sell
         }
    }   
      
   // See if we need to close any opened long positions
   
   if (Buy_opened && bearish)
   {
      CTrade *CTptr = new CTrade;
      
      bool posnClosed = CTptr.PositionClose(_Symbol,DEVIATION_PIPS);  // Symbol, Deviation 
      if(posnClosed) //Request is completed or order placed
      {
         Alert("Long position has been closed");
         Print("Long position has been closed");
      }
    }
    
    // See if we need to close any opened short positions

    else if (Sell_opened && bullish)
    {
       CTrade *CTptr = new CTrade;
      
       bool posnClosed = CTptr.PositionClose(_Symbol,DEVIATION_PIPS);  // Symbol, Deviation 
       if(posnClosed) //Request is completed or order placed
       {
          Alert("Short position has been closed");
          Print("Short position has been closed");
       }
    }
     
    // If bullish go long.
    //--------------------
     
    if (bullish && !Buy_opened)
    {
         // load order request structure
         mrequest.action = TRADE_ACTION_DEAL;                                // immediate order execution
         mrequest.price = NormalizeDouble(latest_price.ask,_Digits);          // latest ask price
         mrequest.sl = NormalizeDouble(latest_price.ask - stopLoss*_Point,_Digits); // Stop Loss
         mrequest.tp = NormalizeDouble(latest_price.ask + takeProfit*_Point,_Digits); // Take Profit
         mrequest.symbol = _Symbol;                                         // currency pair
         mrequest.volume = Lot;                                            // number of lots to trade
         mrequest.magic = Magic;                                           // Order Magic Number
         mrequest.type = ORDER_TYPE_BUY;                                     // Buy Order
         mrequest.type_filling = ORDER_FILLING_AON;                          // Order execution type
         mrequest.deviation=DEVIATION_PIPS;                                // Deviation from current price

         //--- send order
         OrderSend(mrequest,mresult);

         // get the result code
         if(mresult.retcode==10009 || mresult.retcode==10008) //Request is completed or order placed
         {
            Alert("A Buy order has been successfully placed with Ticket#:",mresult.order,"!!");
         }
         else
         {
            Alert("The Buy order request could not be completed -error:",GetLastError());
            ResetLastError();           
            return;
         }   
      }
      
     else if (bearish && !Sell_opened)
     {
         // load order request structure
         mrequest.action = TRADE_ACTION_DEAL;                                // immediate order execution
         mrequest.price = NormalizeDouble(latest_price.bid,_Digits);          // latest ask price
         mrequest.sl = NormalizeDouble(latest_price.bid + stopLoss*_Point,_Digits); // Stop Loss
         mrequest.tp = NormalizeDouble(latest_price.bid - takeProfit*_Point,_Digits); // Take Profit
         mrequest.symbol = _Symbol;                                         // currency pair
         mrequest.volume = Lot;                                            // number of lots to trade
         mrequest.magic = Magic;                                           // Order Magic Number
         mrequest.type = ORDER_TYPE_SELL;                                     // Buy Order
         mrequest.type_filling = ORDER_FILLING_AON;                          // Order execution type
         mrequest.deviation=DEVIATION_PIPS;                                // Deviation from current price

         //--- send order
         OrderSend(mrequest,mresult);

         // get the result code
         if(mresult.retcode==10009 || mresult.retcode==10008) //Request is completed or order placed
         {
            Alert("A Sell order has been successfully placed with Ticket#:",mresult.order,"!!");
         }
         else
         {
            Alert("The Sell order request could not be completed -error:",GetLastError());
            ResetLastError();           
            return;
         }   

     }
     
     return;
  }
